unit MainForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, GrammarReader, StdCtrls, GOLDParser, Reduction, Symbol;

type
  TMain = class(TForm)
    cmdParse: TButton;
    GroupBox1: TGroupBox;
    txtTestInput: TMemo;
    Label1: TLabel;
    txtCGTFilePath: TEdit;
    Label2: TLabel;
    chkTrimReductions: TCheckBox;
    GroupBox2: TGroupBox;
    txtParseTree: TMemo;
    cmdClose: TButton;
    OpenDialog1: TOpenDialog;
    cmdOpenFile: TButton;
    procedure cmdParseClick(Sender: TObject);
    procedure DrawReductionTree(TheReduction: TReduction);
    procedure DrawReduction(TheReduction: TReduction; Indent: Integer);
    procedure PrintParseTree(Text: String);
    procedure cmdCloseClick(Sender: TObject);
    procedure cmdOpenFileClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Main: TMain;

implementation

{$R *.dfm}

procedure TMain.cmdParseClick(Sender: TObject);
var
   GP: TGOLDParser;
   txt: string;
   Response: TGPMessageConstants;
   Done: Boolean;
   ReductionNumber: Integer;
   n: Integer;
begin
   txtParseTree.Text := '';
   ReductionNumber := 0;
   GP := TGOLDParser.Create;

   if GP.LoadCompiledGrammar(txtCGTFilePath.Text) then
   begin
      txt := txtTestInput.Text;
      GP.OpenTextString(txt);
      GP.TrimReductions := chkTrimReductions.Checked;
      txtParseTree.Clear;

      Done := False;
      while not Done do
      begin
         Response := GP.Parse;

         case Response of
         gpMsgLexicalError:
            begin
               //AddToReport "Lexical Error", "Cannot recognize token", Parser.CurrentToken.Data, "", Parser.CurrentLineNumber
               txtParseTree.Text := 'Line ' + IntToStr(GP.CurrentLineNumber) + ': Lexical Error: Cannot recognize token: ' + GP.CurrentToken.DataVar;
               Done := True;
            end;
         gpMsgSyntaxError:
            begin
               txt := '';
               for n := 0 to GP.TokenCount - 1 do
                  txt := txt + ' ' + GP.Tokens(n).Name;

               //AddToReport "Syntax Error", "Expecting the following tokens", LTrim(Text), "", Parser.CurrentLineNumber
               txtParseTree.Text := 'Line ' + IntToStr(GP.CurrentLineNumber) + ': Syntax Error: Expecting the following tokens: ' + Trim(txt);
               Done := True;
            end;
         gpMsgReduction:
            begin
               Inc(ReductionNumber);
               GP.CurrentReduction.Tag := ReductionNumber;   //Mark the reduction
               //AddToReport "Reduce", Parser.CurrentReduction.ParentRule.Text, ReductionNumber, Parser.CurrentReduction.ParentRule.TableIndex, Parser.CurrentLineNumber
            end;
         gpMsgAccept:
            begin
               //=== Success!
               //AddToReport "Accept", Parser.CurrentReduction.ParentRule.Text, "", Parser.CurrentReduction.ParentRule.TableIndex, Parser.CurrentLineNumber
               DrawReductionTree(GP.CurrentReduction);
               Done := True;
            end;
         gpMsgTokenRead:
            begin
//                txtParseTree.Lines.Add(GP.CurrentToken.Name +'='+GP.CurrentToken.DataVar)
               //AddToReport('Token Read', Parser.CurrentToken.Name, Parser.CurrentToken.Text, Parser.CurrentToken.TableIndex, Parser.CurrentLineNumber);
            end;
         gpMsgInternalError:
            begin
               //AddToReport "Internal Error", "Something is horribly wrong", "", "", Parser.CurrentLineNumber
               Done := True;
            end;
         gpMsgNotLoadedError:
            begin
               //=== Due to the if-statement above, this case statement should never be true
               //AddToReport "Not Loaded Error", "Compiled Gramar Table not loaded", "", "", 0
               Done := True;
            end;
         gpMsgCommentError:
            begin
               //AddToReport "Comment Error", "Unexpected end of file", "", "", Parser.CurrentLineNumber
               Done := True;
            end;
         end;
      end;
   end else
      ShowMessage('Input file could not be opened!');


   GP.Free;
end;

procedure TMain.DrawReductionTree(TheReduction: TReduction);
begin
    //This procedure starts the recursion that draws the parse tree.
    txtParseTree.Visible := False;   //Keep the system from updating it until we are done
//    txtParseTree.Text := '';

    DrawReduction(TheReduction, 0);

    txtParseTree.Visible := True;
end;

procedure TMain.DrawReduction(TheReduction: TReduction; Indent: Integer);
Const
   kIndentText = '|  ';
var
   n: Integer;
   IndentText: string;
begin
   //This is a simple recursive procedure that draws an ASCII version of the parse
   //tree


   IndentText := '';
   for n := 1 to Indent do
       IndentText := IndentText + kIndentText;


   //==== Display Reduction
   PrintParseTree(IndentText + '+--' + TheReduction.ParentRule.Text);

   //=== Display the children of the reduction
   for n := 0 to TheReduction.TokenCount - 1 do
   begin
       case TheReduction.Tokens[n].Kind of
       SymbolTypeNonterminal:
          DrawReduction(TheReduction.Tokens[n].DataObj, (Indent + 1));
       else
          PrintParseTree(IndentText + kIndentText + '+--' + TheReduction.Tokens[n].DataVar);
       end;
   end;

end;

procedure TMain.PrintParseTree(Text: String);
begin
   //This sub just appends the Text to the end of the txtParseTree textbox.

   txtParseTree.Lines.Add(Text);

end;

procedure TMain.cmdCloseClick(Sender: TObject);
begin

   Application.Terminate;

end;

procedure TMain.cmdOpenFileClick(Sender: TObject);
begin
   if OpenDialog1.Execute then
      txtCGTFilePath.Text := OpenDialog1.FileName;
end;

procedure TMain.FormCreate(Sender: TObject);
begin

   txtCGTFilePath.Text := ExtractFileDir(Application.ExeName) + '\simple.cgt';

   //ShowMessage('Warning: This is the Alpha version of the GOLD Parser Engine Delphi version!'#13#10#13#10'Use it at youre own risk! I am not responsible for any damage wich might have been caused by this program!');

end;

end.
